@@ -9,7 +9,7 @@ import time
 from dataclasses import asdict, astuple, dataclass
 from decimal import Decimal
 
-from mysql.connector import MySQLConnection
+from mysql.connector import MySQLConnection, connect
 from mysql.connector.pooling import MySQLConnectionPool
 
 
@@ -149,14 +149,8 @@ class MysqlTableCompare:
                 self.write_checkpoint()
                 self.logger.info(f"checkpoint: {self.checkpoint}")
 
-            _progress_rate = round(self.processed_rows_number / self.source_table_rows_number * 100, 1)
-
             batch_id += 1
 
-        self.logger.debug(
-            f"compare progress - pending:{len(cts)}, progress:{_progress_rate}%, different: {self.different_rows_number}, total rows: {self.source_table_rows_number}."
-        )
-
         return batch_id
 
     def write_different(self, rows: list[dict]):
@@ -195,7 +189,8 @@ class MysqlTableCompare:
         _stmt = f"SELECT * FROM {database}.{table} WHERE ({', '.join(formatted_cols)}) IN ({in_clause})"
 
         with con.get_connection() as con:
-            with con.cursor(dictionary=True) as cur:
+            con: MySQLConnection
+            with con.cursor(dictionary=True, buffered=True) as cur:
                 cur.execute(_stmt, tuple(params))
                 return cur.fetchall()
 
@@ -217,10 +212,16 @@ class MysqlTableCompare:
                 operator = ">" if i == end_idx else "="
                 condition_parts.append(f"`{column_name}` {operator} %s")
             where_conditions.append(" and ".join(condition_parts))
-        where_clause = "WHERE" + "(" + ") or (".join(where_conditions) + ") "
+        where_clause = "WHERE" + "(" + ") or (".join(where_conditions) + ")"
+
+        statement_with_condition = f"SELECT {_key_colns} FROM {self.src_database}.{self.src_table} {where_clause} ORDER BY {_key_colns} LIMIT {self.limit_size}"
+        statement_without_condition = f"SELECT {_key_colns} FROM {self.src_database}.{self.src_table} ORDER BY {_key_colns} LIMIT {self.limit_size}"
 
         with self.source_conpool.get_connection() as source_con:
+            source_con: MySQLConnection
+
             while True:
+                _start_time = time.time()
                 _params: list = []
 
                 if _keyval:
@@ -228,8 +229,7 @@ class MysqlTableCompare:
                         for i, (column_name, _) in enumerate(keycols[: end_idx + 1]):
                             _params.append(_keyval[column_name])
 
-                _where_clause = where_clause if _keyval else ""
-                statement = f"SELECT {_key_colns} FROM {self.src_database}.{self.src_table} {_where_clause}ORDER BY {_key_colns} LIMIT {self.limit_size}"
+                statement = statement_with_condition if _params else statement_without_condition
 
                 self.logger.debug(f"full table query - sql: {statement}, params: {_params}, {_keyval}")
 
@@ -239,12 +239,15 @@ class MysqlTableCompare:
                     cur.execute(statement, params=_params)
 
                     while True:
+                        _start_time_1 = time.time()
                         rows = cur.fetchmany(size=self.fetch_size)
                         if rows:
                             _keyval = rows[-1]
                             _has_data = True
+                            self.logger.debug(f"full table query - fetch once fetch: {get_elapsed_time(_start_time_1, 2)}s.")
                             yield rows
                         else:
+                            self.logger.debug(f"full table query - fetch once query: {get_elapsed_time(_start_time,2 )}s.")
                             break
 
                 if not _has_data:
@@ -278,7 +281,7 @@ class MysqlTableCompare:
     def compare_full_table(self, checkpoint_row):
         _batch_id = 1
         # batch_id, source_key_vals, diff_rows = _tasks.pop(0)
-        _comparison_task_container: list[ComparisonTask] = []
+        _comparison_tasks_container: list[ComparisonTask] = []
 
         fulltasks = enumerate(self.get_full_table_keys_order(self.source_table_keys, checkpoint_row), start=1)
 
@@ -295,17 +298,36 @@ class MysqlTableCompare:
 
             futures = generator_futures(self.parallel)
 
+            last_proc_rows = 0
+            last_snapshot_time = time.time()
+
             while futures:
                 done, _ = concurrent.futures.wait(futures, return_when=concurrent.futures.FIRST_COMPLETED)
 
                 for fut in done:
-                    _comparison_task_container.append(fut.result())
+                    _comparison_tasks_container.append(fut.result())
 
                 futures += generator_futures(len(done))
 
                 futures = [fut for fut in futures if fut not in done]
 
-                _batch_id = self.processing_result(_batch_id, _comparison_task_container)
+                _batch_id = self.processing_result(_batch_id, _comparison_tasks_container)
+
+                cur_time = time.time()
+                elapsed_time = cur_time - last_snapshot_time
+                if elapsed_time >= 2:
+                    proc_rows_delta = self.processed_rows_number - last_proc_rows
+                    avg_rate = round(proc_rows_delta / elapsed_time, 0)
+                    last_proc_rows, last_snapshot_time = self.processed_rows_number, cur_time
+
+                    self.logger.debug(
+                        (
+                            f"compare progress - done:{len(done)}, avg:{avg_rate}, "
+                            f"progress:{round(self.processed_rows_number / self.source_table_rows_number * 100, 1)}%, "
+                            f"different: {self.different_rows_number}, "
+                            f"total rows: {self.source_table_rows_number}."
+                        )
+                    )
 
     def run(self) -> None:
         if os.path.exists(self.done_file):
@@ -313,13 +335,9 @@ class MysqlTableCompare:
 
         self.logger = init_logger(self.compare_name)
 
-        self.logger.info(f"source table connect pool create.")
-        self.source_conpool = MySQLConnectionPool(pool_name="source_conpool", pool_size=math.ceil(self.parallel * 1.2) + 1, **self.source_dsn)
+        self.logger.info("init check.")
 
-        self.logger.info(f"target table connect pool create.")
-        self.target_conpool = MySQLConnectionPool(pool_name="target_conpool", pool_size=math.ceil(self.parallel * 1.2), **self.target_dsn)
-
-        with self.source_conpool.get_connection() as source_con, self.target_conpool.get_connection() as target_con:
+        with connect(**self.source_dsn) as source_con, connect(**self.target_dsn) as target_con:
             source_table_struct: list[tuple[str, str]] = get_table_structure(source_con, self.src_database, self.src_table)
             target_table_struct: list[tuple[str, str]] = get_table_structure(target_con, self.dst_database, self.dst_table)
 
@@ -345,6 +363,11 @@ class MysqlTableCompare:
 
         self.logger.info(f"from checkpoint: {self.checkpoint}")
 
+        self.logger.info(f"source table connect pool create.")
+        self.source_conpool = MySQLConnectionPool(pool_name="source_conpool", pool_size=math.ceil(self.parallel * 1.2) + 1, **self.source_dsn)
+
+        self.logger.info(f"target table connect pool create.")
+        self.target_conpool = MySQLConnectionPool(pool_name="target_conpool", pool_size=math.ceil(self.parallel * 1.2), **self.target_dsn)
         self.compare_full_table(self.checkpoint.row)  # main
 
         self.logger.info(
@@ -366,4 +389,6 @@ if __name__ == "__main__":
         "merchant_center_vela_v1",
         "mc_products_to_tags",
         10,
+        40000,
+        1000,
     ).run()
